@@ -22,7 +22,6 @@ import sys
 import time
 import uuid
 
-from positional import positional
 import requests
 import six
 from six.moves import urllib
@@ -253,7 +252,6 @@ class Session(object):
 
     _DEFAULT_REDIRECT_LIMIT = 30
 
-    @positional(2)
     def __init__(self, auth=None, session=None, original_ip=None, verify=True,
                  cert=None, timeout=None, user_agent=None,
                  redirect=_DEFAULT_REDIRECT_LIMIT, additional_headers=None,
@@ -326,7 +324,6 @@ class Session(object):
             return (header[0], '{SHA1}%s' % token_hash)
         return header
 
-    @positional()
     def _http_log_request(self, url, method=None, data=None,
                           json=None, headers=None, query_params=None,
                           logger=_logger):
@@ -374,7 +371,6 @@ class Session(object):
 
         logger.debug(' '.join(string_parts))
 
-    @positional()
     def _http_log_response(self, response=None, json=None,
                            status_code=None, headers=None, text=None,
                            logger=_logger):
@@ -465,13 +461,12 @@ class Session(object):
         for h in header_names:
             headers.setdefault(h, microversion)
 
-    @positional()
     def request(self, url, method, json=None, original_ip=None,
                 user_agent=None, redirect=None, authenticated=None,
                 endpoint_filter=None, auth=None, requests_auth=None,
                 raise_exc=True, allow_reauth=True, log=True,
                 endpoint_override=None, connect_retries=0, logger=_logger,
-                allow={}, client_name=None, client_version=None,
+                allow=None, client_name=None, client_version=None,
                 microversion=None, microversion_service_type=None,
                 **kwargs):
         """Send an HTTP request with the specified characteristics.
@@ -628,14 +623,14 @@ class Session(object):
             elif self.app_name:
                 agent.append(self.app_name)
 
-            for additional in self.additional_user_agent:
-                agent.append('%s/%s' % additional)
-
             if client_name and client_version:
                 agent.append('%s/%s' % (client_name, client_version))
             elif client_name:
                 agent.append(client_name)
 
+            for additional in self.additional_user_agent:
+                agent.append('%s/%s' % additional)
+
             if not agent:
                 # NOTE(jamielennox): determine_user_agent will return an empty
                 # string on failure so checking for None will ensure it is only
@@ -963,6 +958,22 @@ class Session(object):
         auth = self._auth_required(auth, 'determine endpoint URL')
         return auth.get_endpoint_data(self, **kwargs)
 
+    def get_api_major_version(self, auth=None, **kwargs):
+        """Get the major API version as provided by the auth plugin.
+
+        :param auth: The auth plugin to use for token. Overrides the plugin on
+                     the session. (optional)
+        :type auth: keystoneauth1.plugin.BaseAuthPlugin
+
+        :raises keystoneauth1.exceptions.auth_plugins.MissingAuthPlugin: if a
+            plugin is not available.
+
+        :return: The major version of the API of the service discovered.
+        :rtype: tuple or None
+        """
+        auth = self._auth_required(auth, 'determine endpoint URL')
+        return auth.get_api_major_version(self, **kwargs)
+
     def get_auth_connection_params(self, auth=None, **kwargs):
         """Return auth connection params as provided by the auth plugin.
 
